/*******************************************************************************
***        module: mmp_leddemo_helpers.c
***            by: Jack Eberson
***      compiler: MikroC PRO for ARM V6.0.0
***      hardware: Mikromedia plus with STM32F746ZG and shield
***   description: (display) helper routines
*******************************************************************************/

#include "mmp_leddemo_objects.h"
#include "mmp_leddemo_resources.h"
#include "types.h"

uint8_t UpdateCanData;

// Draw the led in the display.
void hlpShowLed(TCircle *ACircle)
{
   DrawCircle(ACircle);
}

// Draw a text Label in the display
void hlpShowLabel(TLabel *ALabel)
{
   DrawLabel(ALabel) ;
}

// RED led in display ON
void hlpRedLedOn ()
{
  LedRedIndicator.Visible              = 1;
  LedRedIndicator.Active               = 1;
  LedRedIndicator.Transparent          = 1;
  LedRedIndicator.Gradient             = 0;
  LedRedIndicator.Color                = CL_RED;
  hlpShowLed(&LedRedIndicator);
}

// RED led in display OFF
void hlpRedLedOff ()
{
  LedRedIndicator.Visible              = 1;
  LedRedIndicator.Active               = 1;
  LedRedIndicator.Transparent          = 1;
  LedRedIndicator.Gradient             = 0;
  LedRedIndicator.Color                = home.Color; // <-- simulate transparancy
  hlpShowLed(&LedRedIndicator);
}

// GREEN led in display ON
void hlpGreenLedOn ()
{
  LedGreenIndicator.Visible              = 1;
  LedGreenIndicator.Active               = 1;
  LedGreenIndicator.Transparent          = 1;
  LedGreenIndicator.Gradient             = 0;
  LedGreenIndicator.Color                = CL_LIME;
  hlpShowLed(&LedGreenIndicator);
}

// GREEN led in display OFF
void hlpGreenLedOff ()
{
  LedGreenIndicator.Visible              = 1;
  LedGreenIndicator.Active               = 1;
  LedGreenIndicator.Transparent          = 1;
  LedGreenIndicator.Gradient             = 0;
  LedGreenIndicator.Color                = home.Color; // <-- simulate transparancy
  hlpShowLed(&LedGreenIndicator);
}

// BLUE led in display ON
void hlpBlueLedOn ()
{
  LedBlueIndicator.Visible              = 1;
  LedBlueIndicator.Active               = 1;
  LedBlueIndicator.Transparent          = 1;
  LedBlueIndicator.Gradient             = 0;
  LedBlueIndicator.Color                = CL_BLUE;
  hlpShowLed(&LedBlueIndicator);
}

// BLUE led in display OFF
void hlpBlueLedOff ()
{
  LedBlueIndicator.Visible              = 1;
  LedBlueIndicator.Active               = 1;
  LedBlueIndicator.Transparent          = 1;
  LedBlueIndicator.Gradient             = 0;
  LedBlueIndicator.Color                = home.Color; // <-- simulate transparancy
  hlpShowLed(&LedBlueIndicator);
}

// CAN TX led in display ON
void hlpCAN_TxIndicatorOn ()
{
  CANTXIndicator.Visible              = 1;
  CANTXIndicator.Active               = 1;
  CANTXIndicator.Transparent          = 1;
  CANTXIndicator.Gradient             = 0;
  CANTXIndicator.Color                = CL_YELLOW;
  hlpShowLed(&CANTXIndicator);
}

// CAN TX led in display OFF
void hlpCAN_TxIndicatorOFF ()
{
  CANTXIndicator.Visible              = 1;
  CANTXIndicator.Active               = 1;
  CANTXIndicator.Transparent          = 1;
  CANTXIndicator.Gradient             = 0;
  CANTXIndicator.Color                = Box1.Gradient_End_Color; // <-- simulate transparancy
  hlpShowLed(&CANTXIndicator);
}

// CAN RX led in display ON
void hlpCAN_RxIndicatorOn ()
{
  CANRXIndicator.Visible              = 1;
  CANRXIndicator.Active               = 1;
  CANRXIndicator.Transparent          = 1;
  CANRXIndicator.Gradient             = 0;
  CANRXIndicator.Color                = CL_YELLOW;
  hlpShowLed(&CANRXIndicator);
}

// CAN RX led in display OFF
void hlpCAN_RxIndicatorOFF ()
{
  CANRXIndicator.Visible              = 1;
  CANRXIndicator.Active               = 1;
  CANRXIndicator.Transparent          = 1;
  CANRXIndicator.Gradient             = 0;
  CANRXIndicator.Color                = Box1.Gradient_End_Color; // <-- simulate transparancy
  hlpShowLed(&CANRXIndicator);
}

// Write received ID in the display
void hlpCAN_WriteID(uint32_t *ID)
{
  uint8_t tmpBuffer[20];

  LongWordToHex(*ID,tmpBuffer);
  rcvdID.Visible         = 1;
  rcvdID.Active          = 1;
  rcvdID.Caption         = tmpBuffer;
  rcvdID.FontName        = Tahoma11x13_Regular;
  rcvdID.Font_Color      = 0x0010;
  rcvdID.VerticalText    = 0;
  hlpShowLabel(&rcvdID);
}

// Write received DLC in the display
void hlpCAN_WriteDLC(uint8_t *DLC)
{
  uint8_t tmpBuffer[10];

  ByteToHex(*DLC,tmpBuffer);
  rcvdDLC.Visible         = 1;
  rcvdDLC.Active          = 1;
  rcvdDLC.Caption         = tmpBuffer;
  rcvdDLC.FontName        = Tahoma11x13_Regular;
  rcvdDLC.Font_Color      = 0x0010;
  rcvdDLC.VerticalText    = 0;
  hlpShowLabel(&rcvdDLC);
}

// Write received DATA in the display
// Non valid bytes are display as "--".
void hlpCAN_WriteDATA(uint8_t *DLC, uint8_t *DATA)
{
  uint8_t tmpBuffer[20];
  uint8_t i;
  TLabel *DataByte;

  for (i=0;i<*DLC;++i)
  {
     ByteToHex(DATA[i], tmpBuffer);
     switch (i)
     {
       case 1 : DataByte = &rcvdDB1; break;
       case 2 : DataByte = &rcvdDB2; break;
       case 3 : DataByte = &rcvdDB3; break;
       case 4 : DataByte = &rcvdDB4; break;
       case 5 : DataByte = &rcvdDB5; break;
       case 6 : DataByte = &rcvdDB6; break;
       case 7 : DataByte = &rcvdDB7; break;
       default: DataByte = &rcvdDB0;
     }

     DataByte->Visible         = 1;
     DataByte->Active          = 1;
     DataByte->Caption         = tmpBuffer;
     DataByte->FontName        = Tahoma11x13_Regular;
     DataByte->Font_Color      = 0x0010;
     DataByte->VerticalText    = 0;
     hlpShowLabel(DataByte);
  }

  for (i=*DLC;i<8;++i)
  {
     switch (i)
     {
       case 1 : DataByte = &rcvdDB1; break;
       case 2 : DataByte = &rcvdDB2; break;
       case 3 : DataByte = &rcvdDB3; break;
       case 4 : DataByte = &rcvdDB4; break;
       case 5 : DataByte = &rcvdDB5; break;
       case 6 : DataByte = &rcvdDB6; break;
       case 7 : DataByte = &rcvdDB7; break;
       default: DataByte = &rcvdDB0;
     }
     
     DataByte->Visible         = 1;
     DataByte->Active          = 1;
     DataByte->Caption         = "--";
     DataByte->FontName        = Tahoma11x13_Regular;
     DataByte->Font_Color      = 0x0010;
     DataByte->VerticalText    = 0;
     hlpShowLabel(DataByte);
  }
}

// Update the received CAN data in the display

void hlpDisplayReceivedCAN_Data(uint32_t *ID, uint8_t *DLC, uint8_t *Data)
{
    DrawBox(&dynamicbox_1);  // <-- clears the previous text
    hlpCAN_WriteID(ID);
    hlpCAN_WriteDLC(DLC);
    hlpCAN_WriteDATA(DLC, Data);
}

// ====================================
// ==== Value input Screen handings ===
// ====================================

// Clear a label
void _ClearLbl( TLabel *_label) 
{
    TFT_Set_Font(_label->FontName, /*background color of display*/ Box_display.Color, FO_HORIZONTAL);
    TFT_Write_Text(_label->Caption, _label->Left, _label->Top);
}


void UpdateBlinkspeed_RED()
{
   uint8_t tmpBuffer[25];
 
   LongLongSignedToStr(BlinkingSpeedRed.speed, tmpBuffer);
   strcpy(l_blinkspeed_red.Caption, LTrim(tmpBuffer));
   DrawBox(&l_blinkspeed_red_box);
   DrawLabel(&l_blinkspeed_red);
}

void UpdateBlinkspeed_Green()
{
   uint8_t tmpBuffer[25];

   LongLongSignedToStr(BlinkingSpeedGreen.speed, tmpBuffer);
   strcpy(l_blinkspeed_green.Caption, LTrim(tmpBuffer));
   DrawBox(&l_blinkspeed_green_box);
   DrawLabel(&l_blinkspeed_green);
}

void UpdateBlinkspeed_Blue()
{
   uint8_t tmpBuffer[25];

   LongLongSignedToStr(BlinkingSpeedBlue.speed, tmpBuffer);
   strcpy(l_blinkspeed_blue.Caption, LTrim(tmpBuffer));
   DrawBox(&l_blinkspeed_blue_box);
   DrawLabel(&l_blinkspeed_blue);
}


void hlpClearValueInInputField(TLabel *Value, uint16_t *BackGroundColor)
{
    // Clear text in display
    _ClearLbl(Value);
    strcpy(Value->Caption, "_");
    
    // New text in display
    DrawLabel(Value) ;
}

void hlpDeleteLastCharacterInInputField(TLabel *Value, uint16_t *BackGroundColor)
{
    uint8_t i;
    uint8_t *p;
    
    // Clear text in display
    _ClearLbl(Value);
    p = Value->Caption;
    i = strlen(p);
    if (i>1)
    {
       // Move the trailing '_' character one to the left.
       *(p+i-2) = '_';
       *(p+i-1) = '\0';
    }
    
    // Write new text.
    DrawLabel(Value) ;
}

void hlpAcceptValueInInputField(uint8_t *Value)
{
    uint8_t i;

    // Save the final InputScreen string in the 'host' Label...
    strcpy(Value,l_ValueInDisplay.Caption);
  
    // Remove the trailing '_'
    if (strlen(Value) > 0)
    {
      i = strlen(Value)-1;
      *(Value+i) = '\0';
      Value = LTrim(RTrim(Value));
    }
}

void hlpAddCharacterInInputField(TLabel *Value, uint8_t *Character, uint8_t maxlength, uint16_t BackGroundColor)
{
    uint8_t i;
    uint8_t tmpBuffer[50];

    i = strlen(Value->Caption);
    if (i<maxlength)
    {
       // Copy original display string
       strcpy (tmpBuffer, Value->Caption);
       
       // Clear display
       _ClearLbl(Value);

       // Remove last '_' character
       i = strlen(tmpBuffer);
       --i;
       tmpBuffer[i] = '\0';

       // Add new character
       strcat(tmpBuffer, Character);

       // Add '_' character
       strcat(tmpBuffer, "_");

       // Write new text.
       strcpy(Value->Caption,tmpBuffer);
       DrawLabel(Value) ;
    }
}

void hlpToggleMinusCharacterInInputField(TLabel *Value, uint16_t BackGroundColor)
{
    uint8_t *p;
    uint8_t tmpBuffer[50];

    // Clear display
    _ClearLbl(Value);
    
    p = Value->Caption;
    strcpy(tmpBuffer, p);
    
    // preceeding '-' character?
    if (*p == '-')
    {
        strcpy(tmpBuffer, p+1);
        strcpy(p, tmpBuffer);
    }
    else
    {
        *p = '-';
        strcpy(p+1, tmpBuffer);
    }
    
    // Write new text.
    DrawLabel(Value) ;
}