#ifndef __LED_DEMO_OBJECTS_H
#define __LED_DEMO_OBJECTS_H

typedef enum {_taLeft, _taCenter, _taRight} TTextAlign;
typedef enum {_tavTop, _tavMiddle, _tavBottom} TTextAlignVertical;

typedef struct Screen TScreen;

typedef struct {
  unsigned int X_Min;
  unsigned int X_Max;
  unsigned int Y_Min;
  unsigned int Y_Max;
  char         Rotate;
} TTPConstants;

typedef struct  Button {
  TScreen*  OwnerScreen;
  char          Order;
  unsigned int  Left;
  unsigned int  Top;
  unsigned int  Width;
  unsigned int  Height;
  char          Pen_Width;
  unsigned int  Pen_Color;
  char          Visible;
  char          Active;
  char          Transparent;
  char          *Caption;
  TTextAlign    TextAlign;
  TTextAlignVertical TextAlignVertical;
  const char    *FontName;
  unsigned int  Font_Color;
  char          VerticalText;
  char          Gradient;
  char          Gradient_Orientation;
  unsigned int  Gradient_Start_Color;
  unsigned int  Gradient_End_Color;
  unsigned int  Color;
  char          PressColEnabled;
  unsigned int  Press_Color;
  void          (*OnUpPtr)();
  void          (*OnDownPtr)();
  void          (*OnClickPtr)();
  void          (*OnPressPtr)();
} TButton;

typedef struct  Button_Round {
  TScreen*  OwnerScreen;
  char          Order;
  unsigned int  Left;
  unsigned int  Top;
  unsigned int  Width;
  unsigned int  Height;
  char          Pen_Width;
  unsigned int  Pen_Color;
  char          Visible;
  char          Active;
  char          Transparent;
  char          *Caption;
  TTextAlign    TextAlign;
  TTextAlignVertical TextAlignVertical;
  const char    *FontName;
  unsigned int  Font_Color;
  char          VerticalText;
  char          Gradient;
  char          Gradient_Orientation;
  unsigned int  Gradient_Start_Color;
  unsigned int  Gradient_End_Color;
  unsigned int  Color;
  char          Corner_Radius;
  char          PressColEnabled;
  unsigned int  Press_Color;
  void          (*OnUpPtr)();
  void          (*OnDownPtr)();
  void          (*OnClickPtr)();
  void          (*OnPressPtr)();
} TButton_Round;

typedef struct  Label {
  TScreen*  OwnerScreen;
  char          Order;
  unsigned int  Left;
  unsigned int  Top;
  unsigned int  Width;
  unsigned int  Height;
  char          *Caption;
  const char    *FontName;
  unsigned int  Font_Color;
  char          VerticalText;
  char          Visible;
  char          Active;
  void          (*OnUpPtr)();
  void          (*OnDownPtr)();
  void          (*OnClickPtr)();
  void          (*OnPressPtr)();
} TLabel;

typedef struct  Image {
  TScreen*  OwnerScreen;
  char          Order;
  unsigned int  Left;
  unsigned int  Top;
  unsigned int  Width;
  unsigned int  Height;
  const char    *Picture_Name;
  char          Visible;
  char          Active;
  char          Picture_Type;
  char          Picture_Ratio;
  void          (*OnUpPtr)();
  void          (*OnDownPtr)();
  void          (*OnClickPtr)();
  void          (*OnPressPtr)();
} TImage;

typedef struct  Circle {
  TScreen*  OwnerScreen;
  char          Order;
  unsigned int  Left;
  unsigned int  Top;
  unsigned int  Radius;
  char          Pen_Width;
  unsigned int  Pen_Color;
  char          Visible;
  char          Active;
  char          Transparent;
  char          Gradient;
  char          Gradient_Orientation;
  unsigned int  Gradient_Start_Color;
  unsigned int  Gradient_End_Color;
  unsigned int  Color;
  char          PressColEnabled;
  unsigned int  Press_Color;
  void          (*OnUpPtr)();
  void          (*OnDownPtr)();
  void          (*OnClickPtr)();
  void          (*OnPressPtr)();
} TCircle;

typedef struct  Box {
  TScreen*  OwnerScreen;
  char          Order;
  unsigned int  Left;
  unsigned int  Top;
  unsigned int  Width;
  unsigned int  Height;
  char          Pen_Width;
  unsigned int  Pen_Color;
  char          Visible;
  char          Active;
  char          Transparent;
  char          Gradient;
  char          Gradient_Orientation;
  unsigned int  Gradient_Start_Color;
  unsigned int  Gradient_End_Color;
  unsigned int  Color;
  char          PressColEnabled;
  unsigned int  Press_Color;
  void          (*OnUpPtr)();
  void          (*OnDownPtr)();
  void          (*OnClickPtr)();
  void          (*OnPressPtr)();
} TBox;

typedef struct  Box_Round {
  TScreen*  OwnerScreen;
  char          Order;
  unsigned int  Left;
  unsigned int  Top;
  unsigned int  Width;
  unsigned int  Height;
  char          Pen_Width;
  unsigned int  Pen_Color;
  char          Visible;
  char          Active;
  char          Transparent;
  char          Gradient;
  char          Gradient_Orientation;
  unsigned int  Gradient_Start_Color;
  unsigned int  Gradient_End_Color;
  unsigned int  Color;
  char          Corner_Radius;
  char          PressColEnabled;
  unsigned int  Press_Color;
  void          (*OnUpPtr)();
  void          (*OnDownPtr)();
  void          (*OnClickPtr)();
  void          (*OnPressPtr)();
} TBox_Round;

typedef struct  Line {
  TScreen*  OwnerScreen;
  char          Order;
  unsigned int  First_Point_X;
  unsigned int  First_Point_Y;
  unsigned int  Second_Point_X;
  unsigned int  Second_Point_Y;
  char          Pen_Width;
  char          Visible;
  unsigned int  Color;
} TLine;

struct Screen {
  unsigned int           Color;
  unsigned int           Width;
  unsigned int           Height;
  unsigned int           ObjectsCount;
  unsigned int           ButtonsCount;
  TButton                * const code *Buttons;
  unsigned int           Buttons_RoundCount;
  TButton_Round          * const code *Buttons_Round;
  unsigned int           LabelsCount;
  TLabel                 * const code *Labels;
  unsigned int           ImagesCount;
  TImage                 * const code *Images;
  unsigned int           CirclesCount;
  TCircle                * const code *Circles;
  unsigned int           BoxesCount;
  TBox                   * const code *Boxes;
  unsigned int           Boxes_RoundCount;
  TBox_Round             * const code *Boxes_Round;
  unsigned int           LinesCount;
  TLine                  * const code *Lines;
};

#endif
extern   TScreen                home;
extern   TBox                   Box1;
extern   TLabel                 MainLabel;
extern   TButton               bRedOn;
extern   TButton               bRedOff;
extern   TButton               bGreenOn;
extern   TButton               bGreenOff;
extern   TButton               bBlueOn;
extern   TButton               bBlueOff;
extern   TButton               bRedBlink;
extern   TButton               bGreenBlink;
extern   TButton               bBlueBlink;
extern   TLine                  TopLine;
extern   TCircle                LedRedIndicator;
extern   TCircle                LedGreenIndicator;
extern   TCircle                LedBlueIndicator;
extern   TLabel                 l_ID;
extern   TLabel                 l_DLC;
extern   TLabel                 l_DATA;
extern   TCircle                CANRXIndicator;
extern   TLabel                 l_CANRX;
extern   TBox                   dynamicbox_1;
extern   TLabel                 rcvdID;
extern   TLabel                 rcvdDLC;
extern   TLabel                 rcvdDB0;
extern   TLabel                 rcvdDB1;
extern   TLabel                 rcvdDB2;
extern   TLabel                 rcvdDB3;
extern   TLabel                 rcvdDB4;
extern   TLabel                 rcvdDB5;
extern   TLabel                 rcvdDB6;
extern   TLabel                 rcvdDB7;
extern   TCircle                CANTXIndicator;
extern   TLabel                 l_CANTX;
extern   TLabel                 Label5;
extern   TLine                  Line1;
extern   TImage               Image1;
extern   TLabel                 Label6;
extern   TImage               info_button;
extern   TBox                   l_blinkspeed_red_box;
extern   TBox                   l_blinkspeed_green_box;
extern   TBox                   l_blinkspeed_blue_box;
extern   TLabel                 l_blinkspeed_red;
extern   TLabel                 l_blinkspeed_green;
extern   TLabel                 l_blinkspeed_blue;
extern   TLabel                 Label33;
extern   TLabel                 Label34;
extern   TLabel                 Label35;
extern   TLabel                 Label36;
extern   TLine                  Line3;
extern   TLabel                 Label37;
extern   TButton                * const code home_Buttons[9];
extern   TLabel                 * const code home_Labels[26];
extern   TImage                 * const code home_Images[2];
extern   TCircle                * const code home_Circles[5];
extern   TBox                   * const code home_Boxes[5];
extern   TLine                  * const code home_Lines[3];

extern   TScreen                help_screen;
extern   TLabel                 Label7;
extern   TLabel                 Label8;
extern   TLabel                 Label9;
extern   TLabel                 Label10;
extern   TLine                  Line2;
extern   TLabel                 Label11;
extern   TLabel                 Label12;
extern   TLabel                 Label13;
extern   TLabel                 Label14;
extern   TLabel                 Label15;
extern   TLabel                 Label16;
extern   TLabel                 Label20;
extern   TLabel                 Label21;
extern   TLabel                 Label22;
extern   TLabel                 Label23;
extern   TLabel                 Label24;
extern   TLabel                 Label25;
extern   TBox                   Box2;
extern   TImage               help1_backbutton;
extern   TLabel                 Label17;
extern   TLabel                 Label18;
extern   TLabel                 Label19;
extern   TLabel                 Label26;
extern   TLabel                 Label1;
extern   TLabel                 Label2;
extern   TLabel                 Label3;
extern   TLabel                 Label4;
extern   TLabel                 Label27;
extern   TLabel                 Label28;
extern   TLabel                 Label29;
extern   TLabel                 Label30;
extern   TLabel                 Label31;
extern   TLabel                 Label32;
extern   TLabel                 * const code help_screen_Labels[30];
extern   TImage                 * const code help_screen_Images[1];
extern   TBox                   * const code help_screen_Boxes[1];
extern   TLine                  * const code help_screen_Lines[1];

extern   TScreen                ValueInputScreen;
extern   TButton               Button_7;
extern   TButton               Button_4;
extern   TButton               Button_1;
extern   TButton               Button_0;
extern   TButton               Button_8;
extern   TButton               Button_5;
extern   TButton               Button_2;
extern   TBox                   top_bar;
extern   TButton_Round          Button_close;
extern   TButton               Button_plusminus;
extern   TButton               Button_9;
extern   TImage               Button_clear;
extern   TBox_Round             BoxRound1;
extern   TBox_Round             BoxRound3;
extern   TImage               Button_accept;
extern   TBox_Round             BoxRound6;
extern   TImage               Button_backspace;
extern   TButton               Button_dot;
extern   TButton               Button_6;
extern   TButton               Button_3;
extern   TBox_Round             BoxRound2;
extern   TBox_Round             BoxRound4;
extern   TBox_Round             BoxRound7;
extern   TBox_Round             Box_display;
extern   TLabel                 l_input_value_text;
extern   TLabel                 l_input_value_min;
extern   TLabel                 l_input_value_max;
extern   TLabel                 l_ValueInDisplay;
extern   TButton                * const code ValueInputScreen_Buttons[12];
extern   TButton_Round          * const code ValueInputScreen_Buttons_Round[1];
extern   TLabel                 * const code ValueInputScreen_Labels[4];
extern   TImage                 * const code ValueInputScreen_Images[3];
extern   TBox                   * const code ValueInputScreen_Boxes[1];
extern   TBox_Round             * const code ValueInputScreen_Boxes_Round[7];


/////////////////////////
// Events Code Declarations
void Back2Home();
void BlueBlink();
void BlueOff();
void BlueOn();
void GreenBlink();
void GreenOff();
void GreenOn();
void InfoScreen();
void RedBlink();
void RedOff();
void RedOn();
void SetBlinkingSpeedBlue();
void SetBlinkingSpeedGreen();
void SetBlinkingSpeedRed();
void ValueInputScreen_0();
void ValueInputScreen_1();
void ValueInputScreen_2();
void ValueInputScreen_3();
void ValueInputScreen_4();
void ValueInputScreen_5();
void ValueInputScreen_6();
void ValueInputScreen_7();
void ValueInputScreen_8();
void ValueInputScreen_9();
void ValueInputScreen_Accept();
void ValueInputScreen_BackSpace();
void ValueInputScreen_Clear();
void ValueInputScreen_Close();
void ValueInputScreen_dot();
void ValueInputScreen_plusminus();
/////////////////////////

/////////////////////////////////
// Caption variables Declarations
extern char Box1_Caption[];
extern char MainLabel_Caption[];
extern char bRedOn_Caption[];
extern char bRedOff_Caption[];
extern char bGreenOn_Caption[];
extern char bGreenOff_Caption[];
extern char bBlueOn_Caption[];
extern char bBlueOff_Caption[];
extern char bRedBlink_Caption[];
extern char bGreenBlink_Caption[];
extern char bBlueBlink_Caption[];
extern char TopLine_Caption[];
extern char LedRedIndicator_Caption[];
extern char LedGreenIndicator_Caption[];
extern char LedBlueIndicator_Caption[];
extern char l_ID_Caption[];
extern char l_DLC_Caption[];
extern char l_DATA_Caption[];
extern char CANRXIndicator_Caption[];
extern char l_CANRX_Caption[];
extern char dynamicbox_1_Caption[];
extern char rcvdID_Caption[];
extern char rcvdDLC_Caption[];
extern char rcvdDB0_Caption[];
extern char rcvdDB1_Caption[];
extern char rcvdDB2_Caption[];
extern char rcvdDB3_Caption[];
extern char rcvdDB4_Caption[];
extern char rcvdDB5_Caption[];
extern char rcvdDB6_Caption[];
extern char rcvdDB7_Caption[];
extern char CANTXIndicator_Caption[];
extern char l_CANTX_Caption[];
extern char Label5_Caption[];
extern char Line1_Caption[];
extern char Image1_Caption[];
extern char Label6_Caption[];
extern char info_button_Caption[];
extern char l_blinkspeed_red_box_Caption[];
extern char l_blinkspeed_green_box_Caption[];
extern char l_blinkspeed_blue_box_Caption[];
extern char l_blinkspeed_red_Caption[];
extern char l_blinkspeed_green_Caption[];
extern char l_blinkspeed_blue_Caption[];
extern char Label33_Caption[];
extern char Label34_Caption[];
extern char Label35_Caption[];
extern char Label36_Caption[];
extern char Line3_Caption[];
extern char Label37_Caption[];
extern char Label7_Caption[];
extern char Label8_Caption[];
extern char Label9_Caption[];
extern char Label10_Caption[];
extern char Line2_Caption[];
extern char Label11_Caption[];
extern char Label12_Caption[];
extern char Label13_Caption[];
extern char Label14_Caption[];
extern char Label15_Caption[];
extern char Label16_Caption[];
extern char Label20_Caption[];
extern char Label21_Caption[];
extern char Label22_Caption[];
extern char Label23_Caption[];
extern char Label24_Caption[];
extern char Label25_Caption[];
extern char Box2_Caption[];
extern char help1_backbutton_Caption[];
extern char Label17_Caption[];
extern char Label18_Caption[];
extern char Label19_Caption[];
extern char Label26_Caption[];
extern char Label1_Caption[];
extern char Label2_Caption[];
extern char Label3_Caption[];
extern char Label4_Caption[];
extern char Label27_Caption[];
extern char Label28_Caption[];
extern char Label29_Caption[];
extern char Label30_Caption[];
extern char Label31_Caption[];
extern char Label32_Caption[];
extern char Button_7_Caption[];
extern char Button_4_Caption[];
extern char Button_1_Caption[];
extern char Button_0_Caption[];
extern char Button_8_Caption[];
extern char Button_5_Caption[];
extern char Button_2_Caption[];
extern char top_bar_Caption[];
extern char Button_close_Caption[];
extern char Button_plusminus_Caption[];
extern char Button_9_Caption[];
extern char Button_clear_Caption[];
extern char BoxRound1_Caption[];
extern char BoxRound3_Caption[];
extern char Button_accept_Caption[];
extern char BoxRound6_Caption[];
extern char Button_backspace_Caption[];
extern char Button_dot_Caption[];
extern char Button_6_Caption[];
extern char Button_3_Caption[];
extern char BoxRound2_Caption[];
extern char BoxRound4_Caption[];
extern char BoxRound7_Caption[];
extern char Box_display_Caption[];
extern char l_input_value_text_Caption[];
extern char l_input_value_min_Caption[];
extern char l_input_value_max_Caption[];
extern char l_ValueInDisplay_Caption[];
/////////////////////////////////

void DrawScreen(TScreen *aScreen);
void DrawButton(TButton *aButton);
void DrawRoundButton(TButton_Round *Around_button);
void DrawLabel(TLabel *ALabel);
void DrawImage(TImage *AImage);
void DrawCircle(TCircle *ACircle);
void DrawBox(TBox *ABox);
void DrawRoundBox(TBox_Round *Around_box);
void DrawLine(TLine *Aline);
void Check_TP();
void Start_TP();
void Process_TP_Press(unsigned int X, unsigned int Y);
void Process_TP_Up(unsigned int X, unsigned int Y);
void Process_TP_Down(unsigned int X, unsigned int Y);
