/*******************************************************************************
***        module: Timer2.c
***            by: Jack Eberson
***      compiler: MikroC PRO for ARM V6.0.0
***      hardware: Mikromedia plus with STM32F746ZG and shield
***   description: Timer 2 interrupt handler.
***                Interrupt occurs every 50ms @ 216MHz
*******************************************************************************/

// External variables
#include "types.h"

// Nested interrupt vector for Timer 2
void Timer2_interrupt() iv IVT_INT_TIM2
{
  static char next50ms  = 0;
  static char next100ms = 0;
  static char next500ms = 0;
  char index;
  
  TIM2_SR.UIF = 0;       // Clear  interrupt flag

  ++next50ms;
  

  // Handle the 50ms timers...
  for(index=0; index < _T50ms_timers; ++index) if (T50ms[index]) --T50ms[index];

  // 100ms passed?
  if (next50ms >= 2)
  {
    next50ms = 0;
    ++next100ms;

    // Handle all 100ms timers
    for(index=0; index < _T100ms_timers; ++index) if (T100ms[index]) --T100ms[index];

    // 500ms passed?
    if (next100ms >= 5)
    {
       next100ms = 0;
       ++next500ms;
    
       // 1000ms passed?
       if (next500ms >= 2)
       {
          next500ms    = 0;

          // Handle all 1s timers
          for(index=0; index < _T1s_timers; ++index) if (T1s[index]) --T1s[index];
       }

    }
  }
}

// Setup Timer 2: every 50ms @ 216MHz.
// The tool TimerCalculator V4.0.0 has been used to setup the timer 2
void InitTimer2(){
  RCC_APB1ENR.TIM2EN = 1;
  TIM2_CR1.CEN = 0;
  TIM2_PSC = 89;
  TIM2_ARR = 59999;
  NVIC_IntEnable(IVT_INT_TIM2);
  TIM2_DIER.UIE = 1;
  TIM2_CR1.CEN = 1;
}


// Reset all defined software timers
void resetTimers()
{
   uint8_t index = 0;
   for(index=0; index < _T50ms_timers; ++index) T50ms[index]=0;
   for(index=0; index < _T100ms_timers; ++index) T100ms[index]=0;
   for(index=0; index < _T1s_timers; ++index) T1s[index]=0;
}