/*******************************************************************************
***        module: circularbuffer.h
***            by: Jack Eberson
***      compiler: MikroC PRO for ARM V6.0.0
***      hardware: Mikromedia plus with STM32F746ZG
***   description: circular buffer for the CAN messages.
***                This circular buffer software has been found on the internet.
*******************************************************************************/
#include "types.h"

#ifndef __RINGBUFS_H
    #define __RINGBUFS_H
    #define CANBUFFERLENGTH     (uint16_t)10  // <-- length of the circular CAN message buffer
    
    // Define CAN message buffer element
    typedef struct
    {
      uint32_t cmsgID;
      uint8_t  cmsgDLC;
      uint8_t  cmsgData[8];
    }CANmsg_t;
    
    // Define CAN message buffer handler
    typedef struct
    {
      CANmsg_t CANbuffer[CANBUFFERLENGTH];
      uint8_t head;
      uint8_t tail;
      uint8_t count;
    } ringBufS_t;

#endif     

// prototypes
void      ringBufS_init  (ringBufS_t *);
sint16_t  ringBufS_empty (ringBufS_t *);
sint16_t  ringBufS_full  (ringBufS_t *);
sint16_t  ringBufS_pop   (ringBufS_t *, CANmsg_t *);
sint16_t  ringBufS_push  (ringBufS_t *, CANmsg_t *);
void      ringBufS_flush (ringBufS_t *, const int);

extern ringBufS_t ringBufferCAN;
extern CANmsg_t CANmsg;