#include "mmp_leddemo_objects.h"
#include "types.h"
#include "circularbuffer.h"
#include "CAN.h"
#include "mmp_leddemo_helpers.h"

void _HomeScreenActions()
{
    // Handle CANTX led in display
    // The led is triggered in teh CAN-ISR
    if ( (ledstatusCANTX == 0) && (T100ms[_T100ms_CAN_TxIndicator]) )
    {
        ledstatusCANTX = 1;
        hlpCAN_TxIndicatorOn();
    }
    else if ( (ledstatusCANTX == 1) && (T100ms[_T100ms_CAN_TxIndicator] == 0) )
    {
        ledstatusCANTX = 0;
        hlpCAN_TxIndicatorOff();
    }

    // Handle CANRX led in display
    if ( (ledstatusCANRX == 0) && (T100ms[_T100ms_CAN_RxIndicator]) )
    {
        ledstatusCANRX = 1;
        hlpCAN_RxIndicatorOn();
    }
    else if ( (ledstatusCANRX == 1) && (T100ms[_T100ms_CAN_RxIndicator] == 0) )
    {
        ledstatusCANRX = 0;
        hlpCAN_RxIndicatorOff();
    }

    // CAN data available in the buffer?
    if (ringBufS_pop(&ringBufferCAN, &CANmsg) != -1)
    {
       // Handling the appropriate CAN message
       if (CANmsg.cmsgID == 0xAFF1) // <-- blink speed of the RED led
       {
          // Update the received CAN data in the display when the HOME screen is displayed
          UpdateCanData = 1;

          if ((CANmsg.cmsgData[0] >= BlinkingSpeedRed.limit_min) && (CANmsg.cmsgData[0] <= BlinkingSpeedRed.limit_max))
          {
             BlinkingSpeedRed.speed = (uint32_t)CANmsg.cmsgData[0];
             updateBoxblinkspeedRed = 1;  // <-- Update field in HOME Screen
             T100ms[_T100ms_CAN] = 0; // reset CAN write timer to force a CAN message being transmitted
          }
       }
       else if (CANmsg.cmsgID == 0xAFF2) // <-- blink speed of the GREEN led
       {
          // Update the received CAN data in the display when the HOME screen is displayed
          UpdateCanData = 1;

          if ((CANmsg.cmsgData[0] >= BlinkingSpeedGreen.limit_min) && (CANmsg.cmsgData[0] <= BlinkingSpeedGreen.limit_max))
          {
             BlinkingSpeedGreen.speed = (uint32_t)CANmsg.cmsgData[0];
             updateBoxblinkspeedGreen = 1; // <-- Update field in HOME screen
             T100ms[_T100ms_CAN] = 0; // reset CAN write timer to force a CAN message being transmitted
          }
       }
       else if (CANmsg.cmsgID == 0xAFF3) // <-- blink speed of the BLUE led
       {
          // Update the received CAN data in the display when the HOME screen is displayed
          UpdateCanData = 1;

          if ((CANmsg.cmsgData[0] >= BlinkingSpeedBlue.limit_min) && (CANmsg.cmsgData[0] <= BlinkingSpeedBlue.limit_max))
          {
             BlinkingSpeedBlue.speed = (uint32_t)CANmsg.cmsgData[0];
             updateBoxblinkspeedBlue = 1; // <-- Update field in HOME screen
             T100ms[_T100ms_CAN] = 0; // reset CAN write timer to force a CAN message being transmitted
          }
       }
       else if (CANmsg.cmsgID == 0xAFF0) // <-- blink speed defaults for all leds
       {
          // Update the received CAN data in the display when the HOME screen is displayed
          UpdateCanData = 1;

          BlinkingSpeedRed.speed   = (uint32_t)BlinkingSpeedRed.defaultvalue;
          BlinkingSpeedGreen.speed = (uint32_t)BlinkingSpeedGreen.defaultvalue;
          BlinkingSpeedBlue.speed  = (uint32_t)BlinkingSpeedBlue.defaultvalue;
          updateBoxblinkspeedRed   = 1; // <-- Update field in HOME screen
          updateBoxblinkspeedGreen = 1; // <-- Update field in HOME screen
          updateBoxblinkspeedBlue  = 1; // <-- Update field in HOME screen
          T100ms[_T100ms_CAN] = 0; // reset CAN write timer to force a CAN message being transmitted
       }
       else
       {
          // Update the received CAN data in the display when the HOME screen is displayed
          UpdateCanData = 1;
       }
    }
    
    // Update blink speed field RED?
    if (updateBoxblinkspeedRed == 1)
    {
        updateBoxblinkspeedRed = 0;
        UpdateBlinkspeed_RED();
    }
    
    // Update blink speed field GREEN?
    if (updateBoxblinkspeedGreen == 1)
    {
        updateBoxblinkspeedGreen = 0;
        UpdateBlinkspeed_GREEN();
    }
    
    // Update blink speed field BLUE?
    if (updateBoxblinkspeedBlue == 1)
    {
        updateBoxblinkspeedBlue = 0;
        UpdateBlinkspeed_BLUE();
    }
    
    // Update the received CAN data?
    if (UpdateCanData)
    {
       // Update CAN reception info in the display.
       hlpDisplayReceivedCAN_Data(&CANmsg.cmsgID , &CANmsg.cmsgDLC , CANmsg.cmsgData);
       UpdateCanData = 0;
    }

    // Handle blinking of (display) RED led
    if (blinkLED_R)
    {
       if (T100ms[_T100ms_LED_R] == 0)
       {
          LED_R ^= 1;
          T100ms[_T100ms_LED_R] = BlinkingSpeedRed.speed;

          // Update the display led (RED)
          if (LED_R)
          {
            hlpRedLedOn();
          }
          else
          {
            hlpRedLedOff();
          }
       }
    }

    // Handle blinking of (display) GREEN led
    if (blinkLED_G)
    {
       if (T100ms[_T100ms_LED_G] == 0)
       {
          LED_G ^= 1;
          T100ms[_T100ms_LED_G] = BlinkingSpeedGreen.speed;

          // Update the display led (GREEN)
          if (LED_G)
          {
            hlpGreenLedOn();
          }
          else
          {
            hlpGreenLedOff();
          }
       }
    }

    // Handle blinking of (display) BLUE led
    if (blinkLED_B)
    {
       if (T100ms[_T100ms_LED_B] == 0)
       {
          LED_B ^= 1;
          T100ms[_T100ms_LED_B] = BlinkingSpeedBlue.speed;

          // Update the display led (BLUE)
          if (LED_B)
          {
            hlpBlueLedOn();
          }
          else
          {
            hlpBlueLedOff();
          }
       }
    }
}